/* 
 * (c) 2014 Juan Balderas
 * This file is part of Dibuja.

 * Dibuja is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * Dibuja is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Dibuja.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <gtk/gtk.h>
#include <glib/gi18n.h>
#include <locale.h>

#include "config.h"
#include "dibuja.h"
#include "util.h"
#include "color.h"
#include "tools.h"
#include "file.h"
#include "undo.h"
#include "effects.h"
#include "menu.h"
#include "fx-dlg.h"

#define DEF_COMBO_ZOOM_AMT	(6)
#define TARGET_IMG_FILENAME	(0)

static GtkWidget *window;

gboolean on_window1_delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	gboolean ret = TRUE;

	ret = do_quit();

	return ret;
}

GtkWidget *get_widget(const gchar *name)
{
	return GTK_WIDGET(g_object_get_data(G_OBJECT(window), name));
}

/* To fix combo problems in glade */
static void zoom_combo_init(GtkBuilder *builder)
{
	int i = 0;
	char *text[] = {
		"10%",  "25%",  "33%",  "50%",  "67%",   "75%",   "100%",  "200%",
		"300%", "400%", "600%", "800%", "1000%", "1200%", "1600%",  NULL };
	GtkWidget *toolitem = 
					GTK_WIDGET(gtk_builder_get_object(builder, "toolitem1"));
	GtkWidget *combo = gtk_combo_box_entry_new_text();
	GtkEntry *entry;

	for(i = 0; text[i] != NULL; i++){
		gtk_combo_box_append_text(GTK_COMBO_BOX(combo), text[i]);
	}

	gtk_container_add(GTK_CONTAINER(toolitem), combo);
	g_object_set_data(G_OBJECT(window), "combo", combo);
	g_signal_connect(G_OBJECT(combo), "changed", G_CALLBACK(zoom_combo_changed), NULL);
	gtk_combo_box_set_active(GTK_COMBO_BOX(combo), 6);

	entry = GTK_ENTRY(gtk_bin_get_child(GTK_BIN(combo)));
	gtk_entry_set_text(entry, "100%");
	gtk_entry_set_max_length(entry, 6);
	gtk_entry_set_width_chars(entry, 6);
	gtk_entry_set_alignment(entry, .5);
	g_signal_connect(G_OBJECT(entry), "activate",
					 G_CALLBACK(zoom_combo_entry_activate), NULL);
}

void update_fx_menu(GtkBuilder *builder)
{
	GtkWidget *menu;
	char **ops = NULL;
	guint nops = 0;
	int i;

	ops = gegl_list_operations(&nops);

	for(i = 0; i < nops; i++){
		if(strcmp(ops[i], "gegl:grey") == 0){
			menu = GTK_WIDGET (gtk_builder_get_object (builder,
								"menu_effects_grayscale"));
			gtk_widget_show(menu);
		}
		if(strcmp(ops[i], "gegl:invert") == 0){
			menu = GTK_WIDGET (gtk_builder_get_object (builder,
								"menu_effects_invert"));
			gtk_widget_show(menu);
		}
		if(strcmp(ops[i], "gegl:insta-brannan") == 0 ||
		   strcmp(ops[i], "gegl:dibuja-insta-brannan") == 0){
			char *op = NULL;

			menu = GTK_WIDGET (gtk_builder_get_object (builder,
								"menu_effects_insta_brannan"));
		/* The list is alphabetizied, so gegl:insta-brannan (if we have it)
		   will override gegl:dibuja-insta-brannan, which is OK */
			op = g_strdup(ops[i]);

			g_object_set_data_full(G_OBJECT(menu), "op", (gpointer)op, g_free);
			gtk_widget_show(menu);
		}
		if(strcmp(ops[i], "gegl:insta-1977") == 0 ||
		   strcmp(ops[i], "gegl:dibuja-insta-1977") == 0){
			char *op = NULL;

			menu = GTK_WIDGET (gtk_builder_get_object (builder,
								"menu_effects_insta_1977"));
		/* The list is alphabetizied, so gegl:insta-1977 (if we have it)
		   will override gegl:dibuja-insta-1977, which is OK */
			op = g_strdup(ops[i]);

			g_object_set_data_full(G_OBJECT(menu), "op", (gpointer)op, g_free);
			gtk_widget_show(menu);
		}
		if(strcmp(ops[i], "gegl:pixelize") == 0){
			menu = GTK_WIDGET (gtk_builder_get_object (builder,
								"menu_effects_pixelate"));
			gtk_widget_show(menu);
		}
		if(strcmp(ops[i], "gegl:brightness-contrast") == 0){
			menu = GTK_WIDGET (gtk_builder_get_object (builder,
								"menu_effects_brightness_contrast"));
			gtk_widget_show(menu);
		}
		if(strcmp(ops[i], "gegl:gaussian-blur") == 0){
			menu = GTK_WIDGET (gtk_builder_get_object (builder,
								"menu_effects_gaussian_blur"));
			gtk_widget_show(menu);
		}
		if(strcmp(ops[i], "gegl:posterize") == 0){
			menu = GTK_WIDGET (gtk_builder_get_object (builder,
								"menu_effects_posterize"));
			gtk_widget_show(menu);
		}
		if(strcmp(ops[i], "gegl:threshold") == 0){
			menu = GTK_WIDGET (gtk_builder_get_object (builder,
								"menu_effects_threshold"));
			gtk_widget_show(menu);
		}
		if(strcmp(ops[i], "gegl:vignette") == 0){
			menu = GTK_WIDGET (gtk_builder_get_object (builder,
								"menu_effects_vignette"));
			gtk_widget_show(menu);
		}
	}
	if(ops){
		g_free(ops);
	}
}

/* To fix setting widget markup problems in glade */
void on_realize(GtkWidget *widget, gpointer data)
{
	PangoFontDescription *font_desc;
	GtkWidget *tmp = widget;

	if(GTK_IS_BUTTON(tmp)){
		tmp = gtk_bin_get_child(GTK_BIN(widget));
		font_desc = pango_font_description_from_string("Sans 8");
	}
	else{
		font_desc = pango_font_description_from_string("Sans Bold 8");
	}
	gtk_widget_modify_font(tmp, font_desc);
	pango_font_description_free(font_desc);
}

int main(int argc, char *argv[])
{
	GtkWidget *alignment, *canvas;

	GtkBuilder *builder = NULL;
	int ret = 0;
	GError *error = NULL;
	gboolean gegl_initialized = FALSE;

#ifdef ENABLE_NLS
	bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
	textdomain (GETTEXT_PACKAGE);
#endif

	gtk_init (&argc, &argv);

	builder = gtk_builder_new ();
	/* UI_FILE from util.h */
	gtk_builder_add_from_file (builder, UI_FILE, &error);
	if(error != NULL){
		g_message("Dibuja GtkBuilder error: %s.\n"
				  "Exiting...\n", error->message);
		message_box_error("Dibuja GtkBuilder error: %s\n", error->message);
		g_error_free(error);
		goto CLOSE_APP;
	}

	gtk_builder_connect_signals (builder, NULL);

	gtk_icon_theme_append_search_path (gtk_icon_theme_get_default (), ICON_DIR_22x22);
	gtk_icon_theme_append_search_path (gtk_icon_theme_get_default (), ICON_DIR_48x48);

	window = GTK_WIDGET (gtk_builder_get_object (builder, "window1"));
	g_signal_connect(G_OBJECT(window), "destroy", G_CALLBACK(gtk_main_quit), NULL);
	gtk_window_set_icon_from_file(GTK_WINDOW(window), DIBUJA_LOGO, &error);
	if(NULL != error){
		g_message("*** ERROR: %s\n", error->message);
		g_error_free(error);
	}

	gtk_window_set_default_size(GTK_WINDOW(window), 600, 400);

	canvas = dibuja_create_canvas(builder, CANVAS_DEF_WIDTH, CANVAS_DEF_HEIGHT);

	zoom_combo_init(builder);

	alignment = GTK_WIDGET (gtk_builder_get_object (builder, "alignment2"));
	gtk_container_add(GTK_CONTAINER(alignment), canvas);

	util_save_widgets(window, builder);

	color_init(builder, canvas);

	tool_init(builder);

	// TODO: Set line width & brush width from preferences
	gtk_adjustment_set_value(GTK_ADJUSTMENT(gtk_builder_get_object (builder, "adj_line_width")), 1.0);

	// Create undo
	undo_init(SIMPLE_CANVAS(canvas));

	gtk_widget_show_all(window);

/* No pixelize in v1 */
/*#ifdef HAVE_GEGL1
{
	GtkWidget *menu_item = get_widget("menu_effects_pixelate");
	gtk_widget_hide(menu_item);
}
#endif*/

	canvas_set_cursor(canvas, DIBUJA_CURSOR);

	// Open file here 'cause we need the widget realized
	// TODO: Parse cmd line
	if(argc > 1){
		ret = file_open_pixbuf(argv[1]);
	}

	if(!ret){
		dibuja_new_file(NULL, NULL, CANVAS_DEF_WIDTH, CANVAS_DEF_HEIGHT);
	}


	gegl_init (&argc, &argv);
	update_fx_menu(builder);
	gegl_initialized = TRUE;

	if(GTK_IS_BUILDER(builder))
		g_object_unref (builder);

	gtk_main ();

CLOSE_APP:
	if(gegl_initialized)
		gegl_exit ();

	return 0;
}
